#include <bits/stdc++.h>

using namespace std;

#define MAXN 100000

typedef vector<pair<int, int>> lazy_segment_tree;

void segment_tree(lazy_segment_tree& tree, int array_size) {
    int size = 1;
    while(size < array_size) size *= 2;
    tree.resize(size * 2);
}

void lazy_propagation(lazy_segment_tree& tree, int node, int length) {
    if(tree[node].second == 0) return;
    if(length > 1) {
        tree[2*node].second += tree[node].second;
        tree[2*node + 1].second += tree[node].second;
    }
    tree[node].first += tree[node].second * length;
    tree[node].second = 0;
}

void add_util(lazy_segment_tree& tree, int a, int b, int val, int node, int from, int to) {
    if(from > to || from > b || to < a) return;     // out of range

    // [from,to] is contained into [a,b]: update all elements (lazily)
    if(a <= from && to <= b) {
        tree[node].second += val;
        return;
    }
    
    lazy_propagation(tree, node, to - from + 1);
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    add_util(tree, a, b, val, 2*node, from, mid);
    add_util(tree, a, b, val, 2*node + 1, mid + 1, to);
    lazy_propagation(tree, 2*node, mid - from + 1);
    lazy_propagation(tree, 2*node + 1, to - mid);
    tree[node].first = tree[2*node].first + tree[2*node + 1].first;
}

void add(lazy_segment_tree& tree, int a, int b, int val) {
    add_util(tree, a, b, val, 1, 0, tree.size()/2 - 1);
}

int sumq_util(lazy_segment_tree& tree, int a, int b, int node, int from, int to) {
    if(from > to || from > b || to < a) return 0;     // out of range

    lazy_propagation(tree, node, to - from + 1);

    // [from,to] is contained into [a,b]
    if(a <= from && to <= b) return tree[node].first;
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    return sumq_util(tree, a, b, 2*node, from, mid) + sumq_util(tree, a, b, 2*node + 1, mid + 1, to);
}

int sumq(lazy_segment_tree& tree, int a, int b) {
    return sumq_util(tree, a, b, 1, 0, tree.size()/2 - 1);
}


void pivotize(int N, int V[], int outofplace[]) {
    for(int i = 0; i < N; i++) V[i]--;  // start with 0

    // we are gonna use two segment trees, for left and right out-of-place numbers
    lazy_segment_tree left, right;
    segment_tree(left, N);
    segment_tree(right, N);
    for(int i = 0; i < N; i++) {
        outofplace[i] = sumq(left, V[i], V[i]);
        add(left, 0, V[i] - 1, 1);
    }
    for(int i = N-1; i >= 0; i--) {
        outofplace[i] += sumq(right, V[i], V[i]);
        add(right, V[i] + 1, N - 1, 1);
    }
}


int V[MAXN];
int outofplace[MAXN];

int main() {
    FILE *fr, *fw;
    int N, i;

    fr = fopen("input.txt", "r");
    fw = fopen("output.txt", "w");
    assert(1 == fscanf(fr, "%d", &N));
    for(i=0; i<N; i++)
        assert(1 == fscanf(fr, "%d", &V[i]));

    pivotize(N, V, outofplace);

    for(i=0; i<N; i++)
        fprintf(fw, "%d ", outofplace[i]);
    fprintf(fw, "\n");
    fclose(fr);
    fclose(fw);
    return 0;
}
