#include <bits/stdc++.h>

using namespace std;

/* intuitive solution (69 points)
struct node {
    int supervisor;
    int wage;
    int increase;
    node(int w = 0, int s = -1) : supervisor(s), wage(w), increase(0) {}
};

int main() {
    FILE* fr = fopen("input.txt", "r");
    FILE* fw = fopen("output.txt", "w");
    
    int N, M;
    fscanf(fr, "%d %d", &N, &M);
    
    vector<node> emp(N);
    fscanf(fr, "%d", &emp[0].wage);
    for(int i = 1; i < N; i++) fscanf(fr, "%d %d", &emp[i].wage, &emp[i].supervisor);
    
    char c;
    int A, X;
    for(int i = 0; i < M; i++) {
        do{
            fscanf(fr, "%c", &c);
        }while(c != 'p' && c != 'u');

        if(c == 'p'){
            fscanf(fr, "%d %d", &A, &X);
            emp[A-1].increase += X;
        }
        else {
            fscanf(fr, "%d", &A);
            int res = emp[A-1].wage;
            while(emp[A-1].supervisor != -1) {
                A = emp[A-1].supervisor;
                res += emp[A-1].increase;
            }
            fprintf(fw, "%d\n", res);
        }
    }
    
    fclose(fr);
    fclose(fw);
    
    return 0;
}
*/

typedef vector<pair<int, int>> lazy_segment_tree;

void add(lazy_segment_tree& tree, int k, int val) {
    k += tree.size() / 2;
    tree[k].first += val;
    for(k /= 2; k >= 1; k /= 2) tree[k].first = tree[2*k].first + tree[2*k+1].first;
}

void segment_tree(const vector<int>& array, lazy_segment_tree& tree) {
    int size = 1;
    while(size < array.size()) size *= 2;
    tree.resize(size * 2);
    for(int i = 0; i < array.size(); i++) add(tree, i, array[i]);
}

void lazy_propagation(lazy_segment_tree& tree, int node, int length) {
    if(tree[node].second == 0) return;
    if(length > 1) {
        tree[2*node].second += tree[node].second;
        tree[2*node + 1].second += tree[node].second;
    }
    tree[node].first += tree[node].second * length;
    tree[node].second = 0;
}

void add_util(lazy_segment_tree& tree, int a, int b, int val, int node, int from, int to) {
    if(from > to || from > b || to < a) return;     // out of range

    // [from,to] is contained into [a,b]: update all elements (lazily)
    if(a <= from && to <= b) {
        tree[node].second += val;
        return;
    }
    
    lazy_propagation(tree, node, to - from + 1);
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    add_util(tree, a, b, val, 2*node, from, mid);
    add_util(tree, a, b, val, 2*node + 1, mid + 1, to);
    lazy_propagation(tree, 2*node, mid - from + 1);
    lazy_propagation(tree, 2*node + 1, to - mid);
    tree[node].first = tree[2*node].first + tree[2*node + 1].first;
}

void add(lazy_segment_tree& tree, int a, int b, int val) {
    add_util(tree, a, b, val, 1, 0, tree.size()/2 - 1);
}

int sumq_util(lazy_segment_tree& tree, int a, int b, int node, int from, int to) {
    if(from > to || from > b || to < a) return 0;     // out of range

    lazy_propagation(tree, node, to - from + 1);

    // [from,to] is contained into [a,b]
    if(a <= from && to <= b) return tree[node].first;
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    return sumq_util(tree, a, b, 2*node, from, mid) + sumq_util(tree, a, b, 2*node + 1, mid + 1, to);
}

int sumq(lazy_segment_tree& tree, int a, int b) {
    return sumq_util(tree, a, b, 1, 0, tree.size()/2 - 1);
}

void determine_ranges(int N, int& cnt, vector<int> under[], int id, vector<int>& new_indexes, pair<int, int> ranges[]) {
    new_indexes[id] = cnt++;
    ranges[id].first = cnt;
    for(auto x : under[id]) determine_ranges(N, cnt, under, x, new_indexes, ranges);
    ranges[id].second = cnt - 1;
}

int main() {
    FILE* fr = fopen("input.txt", "r");
    FILE* fw = fopen("output.txt", "w");
    
    int N, M;
    fscanf(fr, "%d %d", &N, &M);
    
    int W[N], S[N] = {0};
    fscanf(fr, "%d", &W[0]);
    for(int i = 1; i < N; i++) {
        fscanf(fr, "%d %d", &W[i], &S[i]);
        S[i]--;
    }
    
    vector<int> under[N];
    for(int i = 1; i < N; i++) under[S[i]].push_back(i);
    
    // reorder nodes so that all children of a node are contiguos elements
    int cnt = 0;
    vector<int> new_indexes(N);
    pair<int,int> ranges[N];
    determine_ranges(N, cnt, under, 0, new_indexes, ranges);
    
    // build a segment tree for wages according to the new order
    vector<int> wage(N);
    for(int i = 0; i < N; i++) wage[new_indexes[i]] = W[i];
    lazy_segment_tree tree;
    segment_tree(wage, tree);
    
    //for(int i = 0; i < N; i++) cout << new_indexes[i] << " (" << wage[new_indexes[i]] << ") " << ranges[i].first << "-" << ranges[i].second << endl;
    
    char c;
    int A, X;
    for(int i = 0; i < M; i++) {
        do{
            fscanf(fr, "%c", &c);
        }while(c != 'p' && c != 'u');

        if(c == 'p'){
            fscanf(fr, "%d %d", &A, &X);
            add(tree, ranges[A-1].first, ranges[A-1].second, X);
        }
        else {
            fscanf(fr, "%d", &A);
            fprintf(fw, "%d\n", sumq(tree, new_indexes[A-1], new_indexes[A-1]));
        }
    }
    
    fclose(fr);
    fclose(fw);
    
    return 0;
}

