#include <bits/stdc++.h>

using namespace std;

// first is the number of heads
// second is a flip to be propagated
typedef vector<pair<int, bool>> lazy_segment_tree;

// start with tails
void segment_tree(int array_size, lazy_segment_tree& tree) {
    int size = 1;
    while(size < array_size) size *= 2;
    tree.resize(size * 2);
}

// propagation just amount to a flip of all coins, and to set lazy propagation for children
void lazy_propagation(lazy_segment_tree& tree, int node, int length) {
    if(!tree[node].second) return;
    if(length > 1) {
        tree[2*node].second ^= true;
        tree[2*node + 1].second ^= true;
    }
    tree[node].second = false;
    tree[node].first = length - tree[node].first;
}

void flip_util(lazy_segment_tree& tree, int a, int b, int node, int from, int to) {
    if(from > to || from > b || to < a) return;     // out of range

    // [from,to] is contained into [a,b]: update all elements (lazily)
    if(a <= from && to <= b) {
        tree[node].second ^= true;
        return;
    }
    
    lazy_propagation(tree, node, to - from + 1);
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    flip_util(tree, a, b, 2*node, from, mid);
    flip_util(tree, a, b, 2*node + 1, mid + 1, to);
    lazy_propagation(tree, 2*node, mid - from + 1);
    lazy_propagation(tree, 2*node + 1, to - mid);
    tree[node].first = tree[2*node].first + tree[2*node + 1].first;
}

void flip(lazy_segment_tree& tree, int a, int b) {
    flip_util(tree, a, b, 1, 0, tree.size()/2 - 1);
}

int headq_util(lazy_segment_tree& tree, int a, int b, int node, int from, int to) {
    if(from > to || from > b || to < a) return 0;     // out of range

    lazy_propagation(tree, node, to - from + 1);

    // [from,to] is contained into [a,b]
    if(a <= from && to <= b) return tree[node].first;
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    return headq_util(tree, a, b, 2*node, from, mid) + headq_util(tree, a, b, 2*node + 1, mid + 1, to);
}

int headq(lazy_segment_tree& tree, int a, int b) {
    return headq_util(tree, a, b, 1, 0, tree.size()/2 - 1);
}

int main() {
    ifstream in("input.txt");
    ofstream out("output.txt");
    
    int N, Q;
    in >> N >> Q;
    
    lazy_segment_tree tree;
    segment_tree(N, tree);

    while(--Q >= 0) {
        int cmd, A, B;
        in >> cmd >> A >> B;
        if(cmd == 0) flip(tree, A, B);
        else out << headq(tree, A, B) << endl;
    }
    
    return 0;
}
