#include <bits/stdc++.h>

using namespace std;

struct data {
    int zeros;
    int ones;
    data(int z = 0, int o = 0) : zeros(z), ones(o) {}
    void rotate(int length) { int twos = length - ones - zeros; ones = zeros; zeros = twos; }
    data operator+(const data& d) { return data(zeros + d.zeros, ones + d.ones); }
};

// second is the increments to be propagated
typedef vector<pair<data, int>> lazy_segment_tree;

void lazy_propagation(lazy_segment_tree& tree, int node, int length) {
    tree[node].second %= 3;
    if(tree[node].second == 0) return;

    // possibly lazily propagate to the children
    if(length > 1) {
        tree[2*node].second += tree[node].second;
        tree[2*node + 1].second += tree[node].second;
    }

    // rotate numbers
    do{
        tree[node].first.rotate(length);
    }while(--tree[node].second > 0);
}

void inc_util(lazy_segment_tree& tree, int a, int b, int node, int from, int to) {
    if(from > to || from > b || to < a) return;     // out of range

    // [from,to] is contained into [a,b]: update all elements (lazily)
    if(a <= from && to <= b) {
        tree[node].second++;
        return;
    }

    lazy_propagation(tree, node, to - from + 1);
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    inc_util(tree, a, b, 2*node, from, mid);
    inc_util(tree, a, b, 2*node + 1, mid + 1, to);
    lazy_propagation(tree, 2*node, mid - from + 1);
    lazy_propagation(tree, 2*node + 1, to - mid);
    tree[node].first = tree[2*node].first + tree[2*node + 1].first;
}

void inc(lazy_segment_tree& tree, int a, int b) {
    inc_util(tree, a, b, 1, 0, tree.size()/2 - 1);
}

int countq_util(lazy_segment_tree& tree, int a, int b, int node, int from, int to) {
    if(from > to || from > b || to < a) return 0;     // out of range

    lazy_propagation(tree, node, to - from + 1);

    // [from,to] is contained into [a,b]
    if(a <= from && to <= b) return tree[node].first.zeros;
    
    // split [from,to] and repeat
    int mid = (from + to) / 2;
    return countq_util(tree, a, b, 2*node, from, mid) + countq_util(tree, a, b, 2*node + 1, mid + 1, to);
}

int countq(lazy_segment_tree& tree, int a, int b) {
    return countq_util(tree, a, b, 1, 0, tree.size()/2 - 1);
}

// start with zeros
void segment_tree(int array_size, lazy_segment_tree& tree) {
    int size = 1;
    while(size < array_size) size *= 2;
    tree.resize(size * 2);
    inc(tree, 0, array_size - 1);
}

int main() {
    ifstream in("input.txt");
    ofstream out("output.txt");
    
    int N, Q;
    in >> N >> Q;
    
    lazy_segment_tree tree;
    segment_tree(N, tree);
    
    while(--Q >= 0) {
        int cmd, A, B;
        in >> cmd >> A >> B;
        if(cmd == 0) inc(tree, A, B);
        else out << countq(tree, A, B) << endl;
    }
    
    return 0;
}
